/*
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights reserved.
 *
 * Author: Pops, TCSDeveloper
 */

namespace TopCoder.Rules.DependencyMap.View
{
    using System;
    using System.IO;
    using System.Drawing;
    using System.Drawing.Drawing2D;

    /// <summary>
    /// This class is an Adapter class between the IGraphics interface and System.Drawing.Graphics.  This class
    /// attempts to simplify the Graphics and offer only those features the IGraphics requires.  This class is
    /// created by the BitMapRenderer in response to the GetGraphics call and will simply proxy the it's calls
    /// to the embedded Graphics class.
    /// </summary>
    public class GraphicsAdapter : IGraphics
    {
        private readonly Graphics graphics = null;
        
        /// <summary>
        /// Creates a new instance of the GraphicsAdapter class.  Does not affect the Graphics state.
        /// </summary>
        /// <exception cref="ArgumentNullException">If graphics is null.</exception>
        public GraphicsAdapter(Graphics graphics)
        {
        	if (graphics == null)
        	{
        		throw new ArgumentNullException("graphics");
        	}
        	
            this.graphics = graphics;
        }
        
        /// <summary>
        /// Implements the DrawPath interface method and will redirect the DrawPath call to the like name method
        /// in the Graphics interface.
        /// </summary>
        /// <param name="pen">The pen to draw with</param>
        /// <param name="path">The path to draw</param>
        /// <exception cref="ArgumentNullException">if either arg is null</exception>
        public void DrawPath(Pen pen, GraphicsPath path)
        {
            if (pen == null)
            {
                throw new ArgumentNullException("pen");
            }
            if (path == null)
            {
                throw new ArgumentNullException("path");
            }
            
            this.graphics.DrawPath(pen, path);
        }

        /// <summary>
        /// Implements the DrawString interface method and will redirect the DrawString call to the like name method
        /// in the Graphics interface.
        /// </summary>
        /// <param name="text">The text to draw</param>
        /// <param name="font">The font to use</param>
        /// <param name="brush">The brush to draw with</param>
        /// <param name="point">The point to draw it</param>
        /// <exception cref="ArgumentNullException">if any parameter is null</exception>
        public void DrawString(string text, Font font, Brush brush, PointF point)
        {
            if (text == null)
            {
                throw new ArgumentNullException("text");
            }
            if (font == null)
            {
                throw new ArgumentNullException("font");
            }
            if (brush == null)
            {
                throw new ArgumentNullException("brush");
            }

            this.graphics.DrawString(text, font, brush, point);
        }

        /// <summary>
        /// Implements the MeasureString interface method and will redirect the MeasureString call to the like
        /// named method in the Graphics interface.
        /// </summary>
        /// <param name="text">The text to measure</param>
        /// <param name="font">The text font</param>
        /// <returns>The size of the text</returns>
        /// <exception cref="ArgumentNullException">if text or font is null</exception>
        public SizeF MeasureString(string text, Font font)
        {
            if (text == null)
            {
                throw new ArgumentNullException("text");
            }
            if (font == null)
            {
                throw new ArgumentNullException("font");
            }

            return this.graphics.MeasureString(text, font);
        }

        /// <summary>
        /// Does nothing - there is no post processing to do.
        /// </summary>
        public void Complete()
        {
        }

        /// <summary>
        /// Disposes the resources used by calling the dispose on the Graphics interface.
        /// </summary>
        public void Dispose()
        {
            this.graphics.Dispose();
        }        
    }

}
