/*
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDeveloper
 */

using System;
using System.IO;
using System.Drawing;
using System.Drawing.Drawing2D;
using NUnit.Framework;

namespace TopCoder.Rules.DependencyMap.View
{

    /// <summary>
    /// Demonstration for the Component.
    ///
    /// Demo is split into two parts. The first part runs roughly the code supplied by the designer.
    /// The second part builds complex dependency map as in the algorithm description.
    ///
    /// Notice the output of the demo are not cleared on purpose. Client can view the files with
    /// an SVG explorer.
    /// </summary>
    [TestFixture]
    public class ComponentDemo
    {

        /// <summary>
        /// Get an oval shape from application.
        /// </summary>
        /// <returns>an oval shape</returns>
        private GraphicsPath GetOvalShape()
        {
            GraphicsPath shape = new GraphicsPath();
            shape.AddCurve(new Point[] {new Point(20, 0), new Point(36, 3), new Point(39, 20)});
            shape.AddCurve(new Point[] {new Point(39, 20), new Point(36, 36), new Point(20, 39)});
            shape.AddCurve(new Point[] {new Point(20, 39), new Point(3, 36), new Point(0, 20)});
            shape.AddCurve(new Point[] {new Point(0, 20), new Point(3, 3), new Point(20, 0)});
            shape.CloseFigure();
            return shape;
        }

        /// <summary>
        /// Get a start shape from application.
        /// </summary>
        /// <returns>a star shape</returns>
        private GraphicsPath GetStarShape()
        {
            GraphicsPath shape = new GraphicsPath();
            shape.AddLine(20, 0, 27, 12);
            shape.AddLine(27, 12, 39, 20);
            shape.AddLine(39, 20, 27, 27);
            shape.AddLine(27, 27, 20, 39);
            shape.AddLine(20, 39, 12, 27);
            shape.AddLine(12, 27, 0, 20);
            shape.AddLine(0, 20, 12, 12);
            shape.CloseFigure();
            return shape;
        }

        /// <summary>
        /// The first demo. Runs the code supplied by the designer from CS.
        /// </summary>
        [Test]
        public void ComponentSpecificationDemo()
        {
            // Create the shape factory
            ShapeFactory shapeFactory = new ShapeFactory();

            // Override a common seen shape with an oval (defined by an application method)
            shapeFactory[ShapeFactory.SeenElement] = GetOvalShape();

            // Add a new shape (defined by an application method)
            int BeingBuilt = 4;
            shapeFactory[BeingBuilt] = GetStarShape();

            // Create the dependencies
            Dependency configManager = new Dependency("Config Manager");

            Dependency spellCheck = new Dependency("Spell Check", BeingBuilt);
            spellCheck.AddDependant(configManager);

            Dependency phoneticPatternMatching = new Dependency("Phonetic Pattern Matching");

            Dependency phoneticSpellCheck = new Dependency("Phonetic Spell Check");
            phoneticSpellCheck.AddDependant(configManager);
            phoneticSpellCheck.AddDependant(spellCheck);
            phoneticSpellCheck.AddDependant(phoneticPatternMatching);

            // Create the Dependency Renderer and add some diagram renderers
            DependencyRenderer renderer = new DependencyRenderer(shapeFactory);
            renderer.AddDiagramRenderer(new SVGRenderer(@"../../test_files"));

            // Render some of the dependencies
            renderer.Render(phoneticSpellCheck);
            renderer.Render(spellCheck);
        }

        /// <summary>
        /// The second demo. Builds complex dependency map as in the algorithm description.
        /// </summary>
        [Test]
        public void ComplexDependencyRendering()
        {
            // Create the dependencies
            Dependency a = new Dependency("Complex Dependency Map");
            Dependency b = new Dependency("B");
            Dependency c = new Dependency("C");
            Dependency d = new Dependency("D");
            Dependency e = new Dependency("E");
            Dependency f = new Dependency("F");
            Dependency g = new Dependency("G");

            // Build the mapping
            a.AddDependant(b);
            a.AddDependant(c);
            b.AddDependant(d);
            b.AddDependant(e);
            b.AddDependant(f);
            e.AddDependant(f);
            f.AddDependant(g);
            f.AddDependant(c);

            // Create the Dependency Renderer and add some diagram renderers
            DependencyRenderer renderer = new DependencyRenderer();
            renderer.AddDiagramRenderer(new SVGRenderer(@"../../test_files"));

            // Render the first dependency
            renderer.Render(a);
        }
        
        /// <summary>
        /// Does a short demonstration of the RenderAll functionality.  You will be able to see 4 files in 
        /// the test_files director, A.svf, B.svf, C.svf, D.svf.  Each will be the root of the corresponding
        /// Dependency created in this demo.
        /// </summary>
        [Test]
        public void RenderAllDemo()
        {
            // Create the dependencies
            Dependency a = new Dependency("A");
            Dependency b = new Dependency("B");
            Dependency c = new Dependency("C");
            Dependency d = new Dependency("D");
            
            a.AddDependant(b);
            a.AddDependant(c);
            b.AddDependant(d);
            c.AddDependant(d);
            
            DependencyRenderer renderer = new DependencyRenderer();
            renderer.AddDiagramRenderer(new SVGRenderer(@"../../test_files"));

            // Render the first dependency
            renderer.RenderAll(a);
        }
    }

}
