/*
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDeveloper
 */

using System;
using NUnit.Framework;
using System.Drawing.Drawing2D;

namespace TopCoder.Rules.DependencyMap.View
{

    /// <summary>
    /// Unit test fixture for ShapeFactory.
    ///
    /// Testing of ShapeFactory includes simple test of the setter and getter as well as the predefined shapes.
    /// </summary>
    [TestFixture]
    public class ShapeFactoryUnitTests
    {

        /// <summary>
        /// The type to work with.
        /// </summary>
        private int type = 18;

        /// <summary>
        /// The shape to work with.
        /// </summary>
        private GraphicsPath shape;

        /// <summary>
        /// ShapeFactory instance to test.
        /// </summary>
        private ShapeFactory factory = null;

        /// <summary>
        /// Create a new factory instance for each test.
        /// </summary>
        [SetUp]
        protected void SetUp()
        {
            factory = new ShapeFactory();

            shape = new GraphicsPath();
            shape.AddLine(14, 27, 43, 77);
            shape.AddLine(43, 77, 21, 30);
            shape.CloseFigure();
        }

        /// <summary>
        /// Create instance.
        /// </summary>
        [Test]
        public void Create()
        {
            factory = new ShapeFactory();
            Assert.IsNotNull(factory[ShapeFactory.RelationshipArrow], "Relationship arrow not set.");
            Assert.IsNotNull(factory[ShapeFactory.UnseenElement], "Unseen element not set.");
            Assert.IsNotNull(factory[ShapeFactory.SeenElement], "Seen element not set.");
        }

        /// <summary>
        /// Test setter indexer.
        /// </summary>
        [Test]
        public void SetterIndexer()
        {
            factory[type] = shape;
            Assert.AreEqual(shape.GetBounds(), factory[type].GetBounds(), "Shape mismatches.");
        }

        /// <summary>
        /// Test setter indexer with invalid type.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void SetterIndexer_InvalidType()
        {
            factory[0] = shape;
        }

        /// <summary>
        /// Test setter indexer with null shape.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void SetterIndexer_NullShape()
        {
            factory[type] = null;
        }

        /// <summary>
        /// Test getter indexer.
        /// </summary>
        [Test]
        public void GetterIndexer()
        {
            Assert.IsNull(factory[type], "Shape returned for unknown type.");

            factory[type] = shape;
            Assert.AreEqual(shape.GetBounds(), factory[type].GetBounds(), "Shape mismatches.");
        }

        /// <summary>
        /// Test getter indexer with invalid type.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void GetterIndexer_InvalidType()
        {
            shape = factory[0];
        }
        /// <summary>
        /// Test RemoveShape().
        /// </summary>
        [Test]
        public void RemoveShape()
        {
            factory[type] = shape;
            factory.RemoveShape(type);
            Assert.IsNull(factory[type], "Shape returned for unknown type.");
        }

        /// <summary>
        /// Test AddShape().
        /// </summary>
        [Test]
        public void AddShape()
        {
            factory.AddShape(type, shape);
            Assert.AreEqual(shape.GetBounds(), factory[type].GetBounds(), "Shape mismatches.");
        }

        /// <summary>
        /// Test AddShape() with invalid type.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void AddShape_InvalidType()
        {
            factory.AddShape(0, shape);
        }

        /// <summary>
        /// Test AddShape() with existing type.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void AddShape_ExistingType()
        {
            factory.AddShape(ShapeFactory.RelationshipArrow, shape);
        }

        /// <summary>
        /// Test AddShape() with null shape.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void AddShape_NullShape()
        {
            factory.AddShape(type, null);
        }

        /// <summary>
        /// Test RemoveShape() with invalid type.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void RemoveShape_InvalidType()
        {
            factory.RemoveShape(0);
        }

        /// <summary>
        /// Test RemoveShape() with predefined type.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void RemoveShape_PredefinedType()
        {
            factory.RemoveShape(ShapeFactory.RelationshipArrow);
        }


    }

}
